#!/usr/bin/env python
"""
Copyright (C) 2017-2018 Intel Corporation.

This software and the related documents are Intel copyrighted materials,
and your use of them is governed by the express license under which they
were provided to you (License). Unless the License provides otherwise,
you may not use, modify, copy, publish, distribute, disclose or transmit
this software or the related documents without Intel's prior written permission.

This software and the related documents are provided as is, with no express or
implied warranties, other than those that are expressly stated in the License.

fwu helper functions for mei driver
"""
import os
import errno
import select
import struct
import mei

UUID = '309dcde8-ccb1-4062-8f78-600115a34327'
VER_SIZE_6 = 48
VER_SIZE_7 = 52
VER_SIZE_8 = 56
VER_SIZE_MAX = max(VER_SIZE_6, VER_SIZE_7, VER_SIZE_8)

def connect(mei_fd):
    '''Connect to FWU client'''
    return mei.connect(mei_fd, UUID)

def req_fw_ver(mei_fd):
    '''Send FW version request'''
    buf_write = struct.pack("I", 0x00000000)
    return os.write(mei_fd, buf_write)

def get_fw_ver_raw(mei_fd):
    '''Read raw FW version'''
    rlist, _, _ = select.select([mei_fd], [], [], 5)
    if not rlist:
        raise IOError(errno.ETIME, os.strerror(errno.ETIME) + "in fwu.get_fw_ver")
    buf_read = os.read(mei_fd, VER_SIZE_MAX)
    return buf_read

def fw_ver_parse(buf_read):
    '''Parse FW version'''
    if len(buf_read) == VER_SIZE_6:
        str_ver = struct.unpack("7I4H4H2H", buf_read)
    elif len(buf_read) == VER_SIZE_7:
        str_ver = struct.unpack("7I4H4H2HI", buf_read)
    elif len(buf_read) == VER_SIZE_8:
        str_ver = struct.unpack("7I4H4H2HII", buf_read)
    else:
        raise IOError(errno.EMSGSIZE, "Wrong version message size")
    return str_ver

def get_fw_ver_str(mei_fd):
    '''Read FW version'''
    buf = get_fw_ver_raw(mei_fd)
    return fw_ver_parse(buf)

def fw_ver(mei_fd):
    '''Obtain FW version'''
    req_fw_ver(mei_fd)
    return get_fw_ver_str(mei_fd)

def fw_ver_as_mkhi(mei_fd):
    '''Obtain FW version formatted as mkhi version'''
    ver = fw_ver(mei_fd)
    return [0, 0, 0, 0, ver[8], ver[7], 0, ver[10], ver[9], 0, 0, 0, 0, 0]
