#!/usr/bin/env python
"""
Copyright (C) 2017-2018 Intel Corporation.

This software and the related documents are Intel copyrighted materials,
and your use of them is governed by the express license under which they
were provided to you (License). Unless the License provides otherwise,
you may not use, modify, copy, publish, distribute, disclose or transmit
this software or the related documents without Intel's prior written permission.

This software and the related documents are provided as is, with no express or
implied warranties, other than those that are expressly stated in the License.

mkhi helper functions for mei driver
"""
import os
import errno
import select
import struct
import mei

UUID = '8e6a6715-9abc-4043-88ef-9e39c6f63e0f'
FIXED_UUID = '55213584-9a29-4916-badf-0fb7ed682aeb'

def connect(mei_fd):
    '''Connect to dynamic MKHI client'''
    return mei.connect(mei_fd, UUID)

def connect_fixed(mei_fd):
    '''Connect to fixed MKHI client'''
    return mei.connect(mei_fd, FIXED_UUID)

VER_SIZE_SPS = 20
VER_SIZE = 28
VER_SIZE_MAX = max(VER_SIZE_SPS, VER_SIZE)
ARB_SIZE = 1076

def req_fw_ver(mei_fd):
    '''Send FW version request'''
    buf_write = struct.pack("I", 0x000002FF)
    return os.write(mei_fd, buf_write)

def fw_ver_parse(buf_read):
    '''Parse FW version'''
    if len(buf_read) == VER_SIZE:
        str_ver = struct.unpack("4BH2B2HH2B2HH2B2H", buf_read)
    elif len(buf_read) == VER_SIZE_SPS:
        str_ver = struct.unpack("4BH2B2HH2B2H", buf_read)
    else:
        raise IOError(errno.EMSGSIZE, "Wrong version message size (%d)" % len(buf_read))
    return str_ver

def get_fw_ver_str(mei_fd):
    '''Read FW version'''
    rlist, _, _ = select.select([mei_fd], [], [], 5)
    if not rlist:
        raise IOError(errno.ETIME, os.strerror(errno.ETIME) + "in mkhi.get_fw_info")
    buf = os.read(mei_fd, VER_SIZE_MAX)
    return fw_ver_parse(buf)

def fw_ver(mei_fd):
    '''Read FW version'''
    req_fw_ver(mei_fd)
    return get_fw_ver_str(mei_fd)
