#!/usr/bin/env python
"""
Copyright (C) 2018 Intel Corporation.

This software and the related documents are Intel copyrighted materials,
and your use of them is governed by the express license under which they
were provided to you (License). Unless the License provides otherwise,
you may not use, modify, copy, publish, distribute, disclose or transmit
this software or the related documents without Intel's prior written permission.

This software and the related documents are provided as is, with no express or
implied warranties, other than those that are expressly stated in the License.

SPS utilities
"""
from sps import lpc
from sps import pci
from sps import mm_heci
from sps import exceptions as sps_ex

def get_fw_sts1():
    ''' returns FW status register #1 content '''
    value = 0
    platform = lpc.get_platform()
    if not platform:
        raise sps_ex.PlatformUnknown()

    result, hide_heci_on_exit = lpc.reveal_heci(platform)
    if result:
        bus = platform.heci_bdf.bus
        dev = platform.heci_bdf.dev
        funct = platform.heci_bdf.funct
        offset = 0x40
        value = pci.read_pci_register(bus, dev, funct, offset)
        if hide_heci_on_exit:
            if not lpc.hide_heci(platform):
                raise sps_ex.HeciHideError()
    else:
        raise sps_ex.HeciRevealError()

    return value

def get_operating_mode():
    ''' gets FW mode of operation from FW STS #1'''
    fw_sts1 = get_fw_sts1()
    shift = 16
    mask = 0xF
    return (fw_sts1 >> shift) & mask

def get_fw_version():
    ''' returns FW version '''
    version = []
    platform = lpc.get_platform()
    if not platform:
        raise sps_ex.PlatformUnknown()

    result, hide_heci_on_exit = lpc.reveal_heci(platform)
    if result:
        version = mm_heci.get_fw_version(platform)
        if not version:
            raise sps_ex.SendReceiveMessageError()
        if hide_heci_on_exit:
            if not lpc.hide_heci(platform):
                raise sps_ex.HeciHideError()
    else:
        raise sps_ex.HeciRevealError()

    return version
