#!/usr/bin/env python
"""
Copyright (C) 2018 Intel Corporation.

This software and the related documents are Intel copyrighted materials,
and your use of them is governed by the express license under which they
were provided to you (License). Unless the License provides otherwise,
you may not use, modify, copy, publish, distribute, disclose or transmit
this software or the related documents without Intel's prior written permission.

This software and the related documents are provided as is, with no express or
implied warranties, other than those that are expressly stated in the License.

Platform recognition by LPC
"""
from sps import pci
import sps.exceptions

VENDOR_ID_INTEL = 0x8086
LPC_PCI_DEVICEID = 0x1F
LPC_PCI_FUNCTION = 0
LPC_RCBA_REG_OFFSET = 0xF0
HECI_HOST_ENABLE_RCRB_OFFSET = 0x3428
RCBA_BASE_ADDR_MASK = 0xFFFFC000
HECI_ENABLEMENT_A1B0_BITMASK = 0x02
COMMAND_REGISTER_OFFSET = 0x04
MEMORY_SPACE_ENABLE_MASK = 0x02

PCI_ACCESS = 1
PCIe_ACCESS = 2
SPS30 = 30
SPS40 = 40
SPS50 = 50

class BDF(object):
    def __init__(self, bus=0x00, dev=0x00, funct=0x00):
        self.bus = bus
        self.dev = dev
        self.funct = funct

class Platform(object):
    def __init__(self, pch_did=0x0000, heci_bdf=None, heci_access=None, sps_gen=None):
        self.pch_did = pch_did
        self.heci_bdf = heci_bdf
        self.heci_access = heci_access
        self.sps_gen = sps_gen

SUPPORTED_PLATFORMS = [
    # SPS 3.0 and older
    Platform(0x1D3A, BDF(0x00, 0x16, 0x00), PCI_ACCESS, SPS30),
    Platform(0x1D40, BDF(0x00, 0x16, 0x00), PCI_ACCESS, SPS30),
    Platform(0x1D42, BDF(0x00, 0x16, 0x00), PCI_ACCESS, SPS30),
    Platform(0x1D44, BDF(0x00, 0x16, 0x00), PCI_ACCESS, SPS30),
    Platform(0x1D46, BDF(0x00, 0x16, 0x00), PCI_ACCESS, SPS30),
    Platform(0x1C3A, BDF(0x00, 0x16, 0x00), PCI_ACCESS, SPS30),
    Platform(0x1C42, BDF(0x00, 0x16, 0x00), PCI_ACCESS, SPS30),
    Platform(0x1C44, BDF(0x00, 0x16, 0x00), PCI_ACCESS, SPS30),
    Platform(0x1C46, BDF(0x00, 0x16, 0x00), PCI_ACCESS, SPS30),
    Platform(0x1C48, BDF(0x00, 0x16, 0x00), PCI_ACCESS, SPS30),
    Platform(0x1C4A, BDF(0x00, 0x16, 0x00), PCI_ACCESS, SPS30),
    Platform(0x1C4C, BDF(0x00, 0x16, 0x00), PCI_ACCESS, SPS30),
    Platform(0x1C4E, BDF(0x00, 0x16, 0x00), PCI_ACCESS, SPS30),
    Platform(0x1C50, BDF(0x00, 0x16, 0x00), PCI_ACCESS, SPS30),
    Platform(0x1C58, BDF(0x00, 0x16, 0x00), PCI_ACCESS, SPS30),
    Platform(0x1C5A, BDF(0x00, 0x16, 0x00), PCI_ACCESS, SPS30),
    Platform(0x1C5C, BDF(0x00, 0x16, 0x00), PCI_ACCESS, SPS30),
    Platform(0x2810, BDF(0x00, 0x16, 0x00), PCI_ACCESS, SPS30),
    Platform(0x2812, BDF(0x00, 0x16, 0x00), PCI_ACCESS, SPS30),
    Platform(0x2814, BDF(0x00, 0x16, 0x00), PCI_ACCESS, SPS30),
    Platform(0x2811, BDF(0x00, 0x16, 0x00), PCI_ACCESS, SPS30),
    Platform(0x2813, BDF(0x00, 0x16, 0x00), PCI_ACCESS, SPS30),
    Platform(0x2815, BDF(0x00, 0x16, 0x00), PCI_ACCESS, SPS30),
    Platform(0x2910, BDF(0x00, 0x16, 0x00), PCI_ACCESS, SPS30),
    Platform(0x2912, BDF(0x00, 0x16, 0x00), PCI_ACCESS, SPS30),
    Platform(0x2914, BDF(0x00, 0x16, 0x00), PCI_ACCESS, SPS30),
    Platform(0x2916, BDF(0x00, 0x16, 0x00), PCI_ACCESS, SPS30),
    Platform(0x2918, BDF(0x00, 0x16, 0x00), PCI_ACCESS, SPS30),
    Platform(0x291F, BDF(0x00, 0x16, 0x00), PCI_ACCESS, SPS30),
    Platform(0x2911, BDF(0x00, 0x16, 0x00), PCI_ACCESS, SPS30),
    Platform(0x2913, BDF(0x00, 0x16, 0x00), PCI_ACCESS, SPS30),
    Platform(0x2915, BDF(0x00, 0x16, 0x00), PCI_ACCESS, SPS30),
    Platform(0x2917, BDF(0x00, 0x16, 0x00), PCI_ACCESS, SPS30),
    Platform(0x2919, BDF(0x00, 0x16, 0x00), PCI_ACCESS, SPS30),
    Platform(0x3A10, BDF(0x00, 0x16, 0x00), PCI_ACCESS, SPS30),
    Platform(0x3A12, BDF(0x00, 0x16, 0x00), PCI_ACCESS, SPS30),
    Platform(0x3A14, BDF(0x00, 0x16, 0x00), PCI_ACCESS, SPS30),
    Platform(0x3A16, BDF(0x00, 0x16, 0x00), PCI_ACCESS, SPS30),
    Platform(0x3A18, BDF(0x00, 0x16, 0x00), PCI_ACCESS, SPS30),
    Platform(0x3A1A, BDF(0x00, 0x16, 0x00), PCI_ACCESS, SPS30),
    Platform(0x3A1E, BDF(0x00, 0x16, 0x00), PCI_ACCESS, SPS30),
    Platform(0x3A11, BDF(0x00, 0x16, 0x00), PCI_ACCESS, SPS30),
    Platform(0x3A13, BDF(0x00, 0x16, 0x00), PCI_ACCESS, SPS30),
    Platform(0x3A14, BDF(0x00, 0x16, 0x00), PCI_ACCESS, SPS30),
    Platform(0x3A17, BDF(0x00, 0x16, 0x00), PCI_ACCESS, SPS30),
    Platform(0x3A19, BDF(0x00, 0x16, 0x00), PCI_ACCESS, SPS30),
    Platform(0x3B00, BDF(0x00, 0x16, 0x00), PCI_ACCESS, SPS30),
    Platform(0x3B02, BDF(0x00, 0x16, 0x00), PCI_ACCESS, SPS30),
    Platform(0x3B04, BDF(0x00, 0x16, 0x00), PCI_ACCESS, SPS30),
    Platform(0x3B06, BDF(0x00, 0x16, 0x00), PCI_ACCESS, SPS30),
    Platform(0x3B08, BDF(0x00, 0x16, 0x00), PCI_ACCESS, SPS30),
    Platform(0x3B0A, BDF(0x00, 0x16, 0x00), PCI_ACCESS, SPS30),
    Platform(0x3B0E, BDF(0x00, 0x16, 0x00), PCI_ACCESS, SPS30),
    Platform(0x3B12, BDF(0x00, 0x16, 0x00), PCI_ACCESS, SPS30),
    Platform(0x3B14, BDF(0x00, 0x16, 0x00), PCI_ACCESS, SPS30),
    Platform(0x3B16, BDF(0x00, 0x16, 0x00), PCI_ACCESS, SPS30),
    Platform(0x3B1E, BDF(0x00, 0x16, 0x00), PCI_ACCESS, SPS30),
    Platform(0x3B01, BDF(0x00, 0x16, 0x00), PCI_ACCESS, SPS30),
    Platform(0x3B03, BDF(0x00, 0x16, 0x00), PCI_ACCESS, SPS30),
    Platform(0x3B05, BDF(0x00, 0x16, 0x00), PCI_ACCESS, SPS30),
    Platform(0x3B07, BDF(0x00, 0x16, 0x00), PCI_ACCESS, SPS30),
    Platform(0x3B09, BDF(0x00, 0x16, 0x00), PCI_ACCESS, SPS30),
    Platform(0x3B0B, BDF(0x00, 0x16, 0x00), PCI_ACCESS, SPS30),
    Platform(0x3B0D, BDF(0x00, 0x16, 0x00), PCI_ACCESS, SPS30),
    Platform(0x3B0F, BDF(0x00, 0x16, 0x00), PCI_ACCESS, SPS30),
    Platform(0x3B0C, BDF(0x00, 0x16, 0x00), PCI_ACCESS, SPS30),
    Platform(0x3B10, BDF(0x00, 0x16, 0x00), PCI_ACCESS, SPS30),
    Platform(0x3B11, BDF(0x00, 0x16, 0x00), PCI_ACCESS, SPS30),
    Platform(0x3B13, BDF(0x00, 0x16, 0x00), PCI_ACCESS, SPS30),
    Platform(0x3B15, BDF(0x00, 0x16, 0x00), PCI_ACCESS, SPS30),
    Platform(0x3B17, BDF(0x00, 0x16, 0x00), PCI_ACCESS, SPS30),
    Platform(0x3B18, BDF(0x00, 0x16, 0x00), PCI_ACCESS, SPS30),
    Platform(0x3B19, BDF(0x00, 0x16, 0x00), PCI_ACCESS, SPS30),
    Platform(0x3B1A, BDF(0x00, 0x16, 0x00), PCI_ACCESS, SPS30),
    Platform(0x3B1B, BDF(0x00, 0x16, 0x00), PCI_ACCESS, SPS30),
    Platform(0x3B1C, BDF(0x00, 0x16, 0x00), PCI_ACCESS, SPS30),
    Platform(0x3B1D, BDF(0x00, 0x16, 0x00), PCI_ACCESS, SPS30),
    Platform(0x3B1F, BDF(0x00, 0x16, 0x00), PCI_ACCESS, SPS30),
    Platform(0x1C41, BDF(0x00, 0x16, 0x00), PCI_ACCESS, SPS30),
    Platform(0x1C43, BDF(0x00, 0x16, 0x00), PCI_ACCESS, SPS30),
    Platform(0x1C47, BDF(0x00, 0x16, 0x00), PCI_ACCESS, SPS30),
    Platform(0x1C49, BDF(0x00, 0x16, 0x00), PCI_ACCESS, SPS30),
    Platform(0x1C4B, BDF(0x00, 0x16, 0x00), PCI_ACCESS, SPS30),
    Platform(0x1C4D, BDF(0x00, 0x16, 0x00), PCI_ACCESS, SPS30),
    Platform(0x1C4F, BDF(0x00, 0x16, 0x00), PCI_ACCESS, SPS30),
    Platform(0x1C59, BDF(0x00, 0x16, 0x00), PCI_ACCESS, SPS30),
    Platform(0x1C42, BDF(0x00, 0x16, 0x00), PCI_ACCESS, SPS30),
    Platform(0x1C44, BDF(0x00, 0x16, 0x00), PCI_ACCESS, SPS30),
    Platform(0x1C46, BDF(0x00, 0x16, 0x00), PCI_ACCESS, SPS30),
    Platform(0x1C48, BDF(0x00, 0x16, 0x00), PCI_ACCESS, SPS30),
    Platform(0x1C4A, BDF(0x00, 0x16, 0x00), PCI_ACCESS, SPS30),
    Platform(0x1C4C, BDF(0x00, 0x16, 0x00), PCI_ACCESS, SPS30),
    Platform(0x1C4E, BDF(0x00, 0x16, 0x00), PCI_ACCESS, SPS30),
    Platform(0x1C50, BDF(0x00, 0x16, 0x00), PCI_ACCESS, SPS30),
    Platform(0x1C58, BDF(0x00, 0x16, 0x00), PCI_ACCESS, SPS30),
    Platform(0x1C5A, BDF(0x00, 0x16, 0x00), PCI_ACCESS, SPS30),
    Platform(0x1C5C, BDF(0x00, 0x16, 0x00), PCI_ACCESS, SPS30),
    Platform(0x1C52, BDF(0x00, 0x16, 0x00), PCI_ACCESS, SPS30),
    Platform(0x1C54, BDF(0x00, 0x16, 0x00), PCI_ACCESS, SPS30),
    Platform(0x1C56, BDF(0x00, 0x16, 0x00), PCI_ACCESS, SPS30),
    Platform(0x1C40, BDF(0x00, 0x16, 0x00), PCI_ACCESS, SPS30),
    Platform(0x1C45, BDF(0x00, 0x16, 0x00), PCI_ACCESS, SPS30),
    Platform(0x1C47, BDF(0x00, 0x16, 0x00), PCI_ACCESS, SPS30),
    Platform(0x1C51, BDF(0x00, 0x16, 0x00), PCI_ACCESS, SPS30),
    Platform(0x1C53, BDF(0x00, 0x16, 0x00), PCI_ACCESS, SPS30),
    Platform(0x1C55, BDF(0x00, 0x16, 0x00), PCI_ACCESS, SPS30),
    Platform(0x1C57, BDF(0x00, 0x16, 0x00), PCI_ACCESS, SPS30),
    Platform(0x1C5B, BDF(0x00, 0x16, 0x00), PCI_ACCESS, SPS30),
    Platform(0x1C5D, BDF(0x00, 0x16, 0x00), PCI_ACCESS, SPS30),
    Platform(0x1C5E, BDF(0x00, 0x16, 0x00), PCI_ACCESS, SPS30),
    Platform(0x1C5F, BDF(0x00, 0x16, 0x00), PCI_ACCESS, SPS30),
    Platform(0x1D40, BDF(0x00, 0x16, 0x00), PCI_ACCESS, SPS30),
    Platform(0x1D41, BDF(0x00, 0x16, 0x00), PCI_ACCESS, SPS30),
    Platform(0x1D42, BDF(0x00, 0x16, 0x00), PCI_ACCESS, SPS30),
    Platform(0x1D43, BDF(0x00, 0x16, 0x00), PCI_ACCESS, SPS30),
    Platform(0x1D44, BDF(0x00, 0x16, 0x00), PCI_ACCESS, SPS30),
    Platform(0x1D45, BDF(0x00, 0x16, 0x00), PCI_ACCESS, SPS30),
    Platform(0x1D46, BDF(0x00, 0x16, 0x00), PCI_ACCESS, SPS30),
    Platform(0x1D47, BDF(0x00, 0x16, 0x00), PCI_ACCESS, SPS30),
    Platform(0x1D48, BDF(0x00, 0x16, 0x00), PCI_ACCESS, SPS30),
    Platform(0x1D49, BDF(0x00, 0x16, 0x00), PCI_ACCESS, SPS30),
    Platform(0x1D4A, BDF(0x00, 0x16, 0x00), PCI_ACCESS, SPS30),
    Platform(0x1D4B, BDF(0x00, 0x16, 0x00), PCI_ACCESS, SPS30),
    Platform(0x1D4C, BDF(0x00, 0x16, 0x00), PCI_ACCESS, SPS30),
    Platform(0x1D4D, BDF(0x00, 0x16, 0x00), PCI_ACCESS, SPS30),
    Platform(0x1D4E, BDF(0x00, 0x16, 0x00), PCI_ACCESS, SPS30),
    Platform(0x1D4F, BDF(0x00, 0x16, 0x00), PCI_ACCESS, SPS30),
    Platform(0x1CC0, BDF(0x00, 0x16, 0x00), PCI_ACCESS, SPS30),
    Platform(0x1CC1, BDF(0x00, 0x16, 0x00), PCI_ACCESS, SPS30),
    Platform(0x1CC2, BDF(0x00, 0x16, 0x00), PCI_ACCESS, SPS30),
    Platform(0x1CC3, BDF(0x00, 0x16, 0x00), PCI_ACCESS, SPS30),
    Platform(0x1CC4, BDF(0x00, 0x16, 0x00), PCI_ACCESS, SPS30),
    Platform(0x1CC5, BDF(0x00, 0x16, 0x00), PCI_ACCESS, SPS30),
    Platform(0x1CC6, BDF(0x00, 0x16, 0x00), PCI_ACCESS, SPS30),
    Platform(0x1CC7, BDF(0x00, 0x16, 0x00), PCI_ACCESS, SPS30),
    Platform(0x1CC8, BDF(0x00, 0x16, 0x00), PCI_ACCESS, SPS30),
    Platform(0x1CC9, BDF(0x00, 0x16, 0x00), PCI_ACCESS, SPS30),
    Platform(0x1CCA, BDF(0x00, 0x16, 0x00), PCI_ACCESS, SPS30),
    Platform(0x1CCB, BDF(0x00, 0x16, 0x00), PCI_ACCESS, SPS30),
    Platform(0x1CCC, BDF(0x00, 0x16, 0x00), PCI_ACCESS, SPS30),
    Platform(0x1CCD, BDF(0x00, 0x16, 0x00), PCI_ACCESS, SPS30),
    Platform(0x1CCE, BDF(0x00, 0x16, 0x00), PCI_ACCESS, SPS30),
    Platform(0x1CCF, BDF(0x00, 0x16, 0x00), PCI_ACCESS, SPS30),
    Platform(0x1CD0, BDF(0x00, 0x16, 0x00), PCI_ACCESS, SPS30),
    Platform(0x1CD1, BDF(0x00, 0x16, 0x00), PCI_ACCESS, SPS30),
    Platform(0x1CD2, BDF(0x00, 0x16, 0x00), PCI_ACCESS, SPS30),
    Platform(0x1CD3, BDF(0x00, 0x16, 0x00), PCI_ACCESS, SPS30),
    Platform(0x1CD4, BDF(0x00, 0x16, 0x00), PCI_ACCESS, SPS30),
    Platform(0x1CD5, BDF(0x00, 0x16, 0x00), PCI_ACCESS, SPS30),
    Platform(0x1CD6, BDF(0x00, 0x16, 0x00), PCI_ACCESS, SPS30),
    Platform(0x1CD7, BDF(0x00, 0x16, 0x00), PCI_ACCESS, SPS30),
    Platform(0x1CD8, BDF(0x00, 0x16, 0x00), PCI_ACCESS, SPS30),
    Platform(0x1CD9, BDF(0x00, 0x16, 0x00), PCI_ACCESS, SPS30),
    Platform(0x1CDA, BDF(0x00, 0x16, 0x00), PCI_ACCESS, SPS30),
    Platform(0x1CDB, BDF(0x00, 0x16, 0x00), PCI_ACCESS, SPS30),
    Platform(0x1CDC, BDF(0x00, 0x16, 0x00), PCI_ACCESS, SPS30),
    Platform(0x1CDD, BDF(0x00, 0x16, 0x00), PCI_ACCESS, SPS30),
    Platform(0x1CDE, BDF(0x00, 0x16, 0x00), PCI_ACCESS, SPS30),
    Platform(0x1CDF, BDF(0x00, 0x16, 0x00), PCI_ACCESS, SPS30),
    Platform(0x1E40, BDF(0x00, 0x16, 0x00), PCI_ACCESS, SPS30),
    Platform(0x1E41, BDF(0x00, 0x16, 0x00), PCI_ACCESS, SPS30),
    Platform(0x1E42, BDF(0x00, 0x16, 0x00), PCI_ACCESS, SPS30),
    Platform(0x1E43, BDF(0x00, 0x16, 0x00), PCI_ACCESS, SPS30),
    Platform(0x1E44, BDF(0x00, 0x16, 0x00), PCI_ACCESS, SPS30),
    Platform(0x1E45, BDF(0x00, 0x16, 0x00), PCI_ACCESS, SPS30),
    Platform(0x1E46, BDF(0x00, 0x16, 0x00), PCI_ACCESS, SPS30),
    Platform(0x1E47, BDF(0x00, 0x16, 0x00), PCI_ACCESS, SPS30),
    Platform(0x1E48, BDF(0x00, 0x16, 0x00), PCI_ACCESS, SPS30),
    Platform(0x1E49, BDF(0x00, 0x16, 0x00), PCI_ACCESS, SPS30),
    Platform(0x1E4A, BDF(0x00, 0x16, 0x00), PCI_ACCESS, SPS30),
    Platform(0x1E4B, BDF(0x00, 0x16, 0x00), PCI_ACCESS, SPS30),
    Platform(0x1E4C, BDF(0x00, 0x16, 0x00), PCI_ACCESS, SPS30),
    Platform(0x1E4D, BDF(0x00, 0x16, 0x00), PCI_ACCESS, SPS30),
    Platform(0x1E4E, BDF(0x00, 0x16, 0x00), PCI_ACCESS, SPS30),
    Platform(0x1E4F, BDF(0x00, 0x16, 0x00), PCI_ACCESS, SPS30),
    Platform(0x1E50, BDF(0x00, 0x16, 0x00), PCI_ACCESS, SPS30),
    Platform(0x1E51, BDF(0x00, 0x16, 0x00), PCI_ACCESS, SPS30),
    Platform(0x1E52, BDF(0x00, 0x16, 0x00), PCI_ACCESS, SPS30),
    Platform(0x1E53, BDF(0x00, 0x16, 0x00), PCI_ACCESS, SPS30),
    Platform(0x1E54, BDF(0x00, 0x16, 0x00), PCI_ACCESS, SPS30),
    Platform(0x1E55, BDF(0x00, 0x16, 0x00), PCI_ACCESS, SPS30),
    Platform(0x1E56, BDF(0x00, 0x16, 0x00), PCI_ACCESS, SPS30),
    Platform(0x1E57, BDF(0x00, 0x16, 0x00), PCI_ACCESS, SPS30),
    Platform(0x1E58, BDF(0x00, 0x16, 0x00), PCI_ACCESS, SPS30),
    Platform(0x1E59, BDF(0x00, 0x16, 0x00), PCI_ACCESS, SPS30),
    Platform(0x1E5A, BDF(0x00, 0x16, 0x00), PCI_ACCESS, SPS30),
    Platform(0x1E5B, BDF(0x00, 0x16, 0x00), PCI_ACCESS, SPS30),
    Platform(0x1E5C, BDF(0x00, 0x16, 0x00), PCI_ACCESS, SPS30),
    Platform(0x1E5D, BDF(0x00, 0x16, 0x00), PCI_ACCESS, SPS30),
    Platform(0x1E5E, BDF(0x00, 0x16, 0x00), PCI_ACCESS, SPS30),
    Platform(0x1E5F, BDF(0x00, 0x16, 0x00), PCI_ACCESS, SPS30),
    Platform(0x8C40, BDF(0x00, 0x16, 0x00), PCI_ACCESS, SPS30),
    Platform(0x8C41, BDF(0x00, 0x16, 0x00), PCI_ACCESS, SPS30),
    Platform(0x8C42, BDF(0x00, 0x16, 0x00), PCI_ACCESS, SPS30),
    Platform(0x8C43, BDF(0x00, 0x16, 0x00), PCI_ACCESS, SPS30),
    Platform(0x8C44, BDF(0x00, 0x16, 0x00), PCI_ACCESS, SPS30),
    Platform(0x8C45, BDF(0x00, 0x16, 0x00), PCI_ACCESS, SPS30),
    Platform(0x8C46, BDF(0x00, 0x16, 0x00), PCI_ACCESS, SPS30),
    Platform(0x8C47, BDF(0x00, 0x16, 0x00), PCI_ACCESS, SPS30),
    Platform(0x8C48, BDF(0x00, 0x16, 0x00), PCI_ACCESS, SPS30),
    Platform(0x8C49, BDF(0x00, 0x16, 0x00), PCI_ACCESS, SPS30),
    Platform(0x8C4A, BDF(0x00, 0x16, 0x00), PCI_ACCESS, SPS30),
    Platform(0x8C4B, BDF(0x00, 0x16, 0x00), PCI_ACCESS, SPS30),
    Platform(0x8C4C, BDF(0x00, 0x16, 0x00), PCI_ACCESS, SPS30),
    Platform(0x8C4D, BDF(0x00, 0x16, 0x00), PCI_ACCESS, SPS30),
    Platform(0x8C4E, BDF(0x00, 0x16, 0x00), PCI_ACCESS, SPS30),
    Platform(0x8C4F, BDF(0x00, 0x16, 0x00), PCI_ACCESS, SPS30),
    Platform(0x8C50, BDF(0x00, 0x16, 0x00), PCI_ACCESS, SPS30),
    Platform(0x8C51, BDF(0x00, 0x16, 0x00), PCI_ACCESS, SPS30),
    Platform(0x8C52, BDF(0x00, 0x16, 0x00), PCI_ACCESS, SPS30),
    Platform(0x8C53, BDF(0x00, 0x16, 0x00), PCI_ACCESS, SPS30),
    Platform(0x8C54, BDF(0x00, 0x16, 0x00), PCI_ACCESS, SPS30),
    Platform(0x8C55, BDF(0x00, 0x16, 0x00), PCI_ACCESS, SPS30),
    Platform(0x8C56, BDF(0x00, 0x16, 0x00), PCI_ACCESS, SPS30),
    Platform(0x8C57, BDF(0x00, 0x16, 0x00), PCI_ACCESS, SPS30),
    Platform(0x8C58, BDF(0x00, 0x16, 0x00), PCI_ACCESS, SPS30),
    Platform(0x8C59, BDF(0x00, 0x16, 0x00), PCI_ACCESS, SPS30),
    Platform(0x8C5A, BDF(0x00, 0x16, 0x00), PCI_ACCESS, SPS30),
    Platform(0x8C5B, BDF(0x00, 0x16, 0x00), PCI_ACCESS, SPS30),
    Platform(0x8C5C, BDF(0x00, 0x16, 0x00), PCI_ACCESS, SPS30),
    Platform(0x8C5D, BDF(0x00, 0x16, 0x00), PCI_ACCESS, SPS30),
    Platform(0x8C5E, BDF(0x00, 0x16, 0x00), PCI_ACCESS, SPS30),
    Platform(0x8C5F, BDF(0x00, 0x16, 0x00), PCI_ACCESS, SPS30),
    Platform(0x8D40, BDF(0x00, 0x16, 0x00), PCI_ACCESS, SPS30),
    Platform(0x8D44, BDF(0x00, 0x16, 0x00), PCI_ACCESS, SPS30),
    Platform(0x8D47, BDF(0x00, 0x16, 0x00), PCI_ACCESS, SPS30),
    Platform(0x8D48, BDF(0x00, 0x16, 0x00), PCI_ACCESS, SPS30),
    Platform(0x2310, BDF(0x00, 0x16, 0x00), PCI_ACCESS, SPS30),
    Platform(0x2390, BDF(0x00, 0x16, 0x00), PCI_ACCESS, SPS30),
    Platform(0x2480, BDF(0x00, 0x16, 0x00), PCI_ACCESS, SPS30),
    Platform(0x25A1, BDF(0x00, 0x16, 0x00), PCI_ACCESS, SPS30),
    Platform(0x2670, BDF(0x00, 0x16, 0x00), PCI_ACCESS, SPS30),
    Platform(0x27BC, BDF(0x00, 0x16, 0x00), PCI_ACCESS, SPS30),
    Platform(0x8CC1, BDF(0x00, 0x16, 0x00), PCI_ACCESS, SPS30),
    Platform(0x8CC2, BDF(0x00, 0x16, 0x00), PCI_ACCESS, SPS30),
    Platform(0x8CC3, BDF(0x00, 0x16, 0x00), PCI_ACCESS, SPS30),
    Platform(0x8CC4, BDF(0x00, 0x16, 0x00), PCI_ACCESS, SPS30),
    Platform(0x8D49, BDF(0x00, 0x16, 0x00), PCI_ACCESS, SPS30),
    Platform(0x8D4A, BDF(0x00, 0x16, 0x00), PCI_ACCESS, SPS30),
    Platform(0x8D4B, BDF(0x00, 0x16, 0x00), PCI_ACCESS, SPS30),
    Platform(0x8D4C, BDF(0x00, 0x16, 0x00), PCI_ACCESS, SPS30),
    Platform(0x8D4D, BDF(0x00, 0x16, 0x00), PCI_ACCESS, SPS30),
    Platform(0x8D4E, BDF(0x00, 0x16, 0x00), PCI_ACCESS, SPS30),
    Platform(0x8D4F, BDF(0x00, 0x16, 0x00), PCI_ACCESS, SPS30),
    Platform(0x9C40, BDF(0x00, 0x16, 0x00), PCI_ACCESS, SPS30),
    Platform(0x9C41, BDF(0x00, 0x16, 0x00), PCI_ACCESS, SPS30),
    Platform(0x9C42, BDF(0x00, 0x16, 0x00), PCI_ACCESS, SPS30),
    Platform(0x9C43, BDF(0x00, 0x16, 0x00), PCI_ACCESS, SPS30),
    Platform(0x9C44, BDF(0x00, 0x16, 0x00), PCI_ACCESS, SPS30),
    Platform(0x9C45, BDF(0x00, 0x16, 0x00), PCI_ACCESS, SPS30),
    Platform(0x9C46, BDF(0x00, 0x16, 0x00), PCI_ACCESS, SPS30),
    Platform(0x9C47, BDF(0x00, 0x16, 0x00), PCI_ACCESS, SPS30),
    Platform(0x9CC1, BDF(0x00, 0x16, 0x00), PCI_ACCESS, SPS30),
    Platform(0x9CC2, BDF(0x00, 0x16, 0x00), PCI_ACCESS, SPS30),
    Platform(0x9CC3, BDF(0x00, 0x16, 0x00), PCI_ACCESS, SPS30),
    Platform(0x9CC5, BDF(0x00, 0x16, 0x00), PCI_ACCESS, SPS30),
    Platform(0x9CC6, BDF(0x00, 0x16, 0x00), PCI_ACCESS, SPS30),
    Platform(0x9CC7, BDF(0x00, 0x16, 0x00), PCI_ACCESS, SPS30),
    Platform(0x9CC9, BDF(0x00, 0x16, 0x00), PCI_ACCESS, SPS30),

    # SPS 4.0
    Platform(0x9d43, BDF(0x00, 0x16, 0x00), PCIe_ACCESS, SPS40),
    Platform(0x9d48, BDF(0x00, 0x16, 0x00), PCIe_ACCESS, SPS40),
    Platform(0xA100, BDF(0x00, 0x16, 0x00), PCIe_ACCESS, SPS40),
    Platform(0xA140, BDF(0x00, 0x16, 0x00), PCIe_ACCESS, SPS40),
    Platform(0xA141, BDF(0x00, 0x16, 0x00), PCIe_ACCESS, SPS40),
    Platform(0xA142, BDF(0x00, 0x16, 0x00), PCIe_ACCESS, SPS40),
    Platform(0xA143, BDF(0x00, 0x16, 0x00), PCIe_ACCESS, SPS40),
    Platform(0xA144, BDF(0x00, 0x16, 0x00), PCIe_ACCESS, SPS40),
    Platform(0xA145, BDF(0x00, 0x16, 0x00), PCIe_ACCESS, SPS40),
    Platform(0xA146, BDF(0x00, 0x16, 0x00), PCIe_ACCESS, SPS40),
    Platform(0xA147, BDF(0x00, 0x16, 0x00), PCIe_ACCESS, SPS40),
    Platform(0xA148, BDF(0x00, 0x16, 0x00), PCIe_ACCESS, SPS40),
    Platform(0xA149, BDF(0x00, 0x16, 0x00), PCIe_ACCESS, SPS40),
    Platform(0xA14A, BDF(0x00, 0x16, 0x00), PCIe_ACCESS, SPS40),
    Platform(0xA14B, BDF(0x00, 0x16, 0x00), PCIe_ACCESS, SPS40),
    Platform(0xA14C, BDF(0x00, 0x16, 0x00), PCIe_ACCESS, SPS40),
    Platform(0xA14D, BDF(0x00, 0x16, 0x00), PCIe_ACCESS, SPS40),
    Platform(0xA14E, BDF(0x00, 0x16, 0x00), PCIe_ACCESS, SPS40),
    Platform(0xA14F, BDF(0x00, 0x16, 0x00), PCIe_ACCESS, SPS40),
    Platform(0xA150, BDF(0x00, 0x16, 0x00), PCIe_ACCESS, SPS40),
    Platform(0xA151, BDF(0x00, 0x16, 0x00), PCIe_ACCESS, SPS40),
    Platform(0xA152, BDF(0x00, 0x16, 0x00), PCIe_ACCESS, SPS40),
    Platform(0xA153, BDF(0x00, 0x16, 0x00), PCIe_ACCESS, SPS40),
    Platform(0xA154, BDF(0x00, 0x16, 0x00), PCIe_ACCESS, SPS40),
    Platform(0xA155, BDF(0x00, 0x16, 0x00), PCIe_ACCESS, SPS40),
    Platform(0xA156, BDF(0x00, 0x16, 0x00), PCIe_ACCESS, SPS40),
    Platform(0xA157, BDF(0x00, 0x16, 0x00), PCIe_ACCESS, SPS40),
    Platform(0xA158, BDF(0x00, 0x16, 0x00), PCIe_ACCESS, SPS40),
    Platform(0xA159, BDF(0x00, 0x16, 0x00), PCIe_ACCESS, SPS40),
    Platform(0xA15A, BDF(0x00, 0x16, 0x00), PCIe_ACCESS, SPS40),
    Platform(0xA15B, BDF(0x00, 0x16, 0x00), PCIe_ACCESS, SPS40),
    Platform(0xA15C, BDF(0x00, 0x16, 0x00), PCIe_ACCESS, SPS40),
    Platform(0xA15D, BDF(0x00, 0x16, 0x00), PCIe_ACCESS, SPS40),
    Platform(0xA15E, BDF(0x00, 0x16, 0x00), PCIe_ACCESS, SPS40),
    Platform(0xA15F, BDF(0x00, 0x16, 0x00), PCIe_ACCESS, SPS40),
    Platform(0x19DC, BDF(0x00, 0x18, 0x00), PCI_ACCESS, SPS40),
    Platform(0xA1C0, BDF(0x00, 0x16, 0x00), PCI_ACCESS, SPS40),
    Platform(0xA1C1, BDF(0x00, 0x16, 0x00), PCI_ACCESS, SPS40),
    Platform(0xA1C2, BDF(0x00, 0x16, 0x00), PCI_ACCESS, SPS40),
    Platform(0xA1C3, BDF(0x00, 0x16, 0x00), PCI_ACCESS, SPS40),
    Platform(0xA1C4, BDF(0x00, 0x16, 0x00), PCI_ACCESS, SPS40),
    Platform(0xA1C5, BDF(0x00, 0x16, 0x00), PCI_ACCESS, SPS40),
    Platform(0xA1C6, BDF(0x00, 0x16, 0x00), PCI_ACCESS, SPS40),
    Platform(0xA1C7, BDF(0x00, 0x16, 0x00), PCI_ACCESS, SPS40),
    Platform(0xA1C8, BDF(0x00, 0x16, 0x00), PCI_ACCESS, SPS40),
    Platform(0xA1C9, BDF(0x00, 0x16, 0x00), PCI_ACCESS, SPS40),
    Platform(0xA1CA, BDF(0x00, 0x16, 0x00), PCI_ACCESS, SPS40),
    Platform(0xA1CB, BDF(0x00, 0x16, 0x00), PCI_ACCESS, SPS40),
    Platform(0xA1CC, BDF(0x00, 0x16, 0x00), PCI_ACCESS, SPS40),
    Platform(0xA1CD, BDF(0x00, 0x16, 0x00), PCI_ACCESS, SPS40),
    Platform(0xA1CE, BDF(0x00, 0x16, 0x00), PCI_ACCESS, SPS40),
    Platform(0xA1CF, BDF(0x00, 0x16, 0x00), PCI_ACCESS, SPS40),
    Platform(0xA240, BDF(0x00, 0x16, 0x00), PCI_ACCESS, SPS40),
    Platform(0xA241, BDF(0x00, 0x16, 0x00), PCI_ACCESS, SPS40),
    Platform(0xA242, BDF(0x00, 0x16, 0x00), PCI_ACCESS, SPS40),
    Platform(0xA243, BDF(0x00, 0x16, 0x00), PCI_ACCESS, SPS40),
    Platform(0xA244, BDF(0x00, 0x16, 0x00), PCI_ACCESS, SPS40),
    Platform(0xA245, BDF(0x00, 0x16, 0x00), PCI_ACCESS, SPS40),
    Platform(0xA246, BDF(0x00, 0x16, 0x00), PCI_ACCESS, SPS40),
    Platform(0xA247, BDF(0x00, 0x16, 0x00), PCI_ACCESS, SPS40),
    Platform(0xA248, BDF(0x00, 0x16, 0x00), PCI_ACCESS, SPS40),
    Platform(0xA249, BDF(0x00, 0x16, 0x00), PCI_ACCESS, SPS40),
    Platform(0xA24A, BDF(0x00, 0x16, 0x00), PCI_ACCESS, SPS40),
    Platform(0xA24B, BDF(0x00, 0x16, 0x00), PCI_ACCESS, SPS40),
    Platform(0xA24C, BDF(0x00, 0x16, 0x00), PCI_ACCESS, SPS40),
    Platform(0xA24D, BDF(0x00, 0x16, 0x00), PCI_ACCESS, SPS40),
    Platform(0xA24E, BDF(0x00, 0x16, 0x00), PCI_ACCESS, SPS40),
    Platform(0xA24F, BDF(0x00, 0x16, 0x00), PCI_ACCESS, SPS40),
    Platform(0xA2C4, BDF(0x00, 0x16, 0x00), PCI_ACCESS, SPS40),
    Platform(0xA2C5, BDF(0x00, 0x16, 0x00), PCI_ACCESS, SPS40),
    Platform(0xA2C6, BDF(0x00, 0x16, 0x00), PCI_ACCESS, SPS40),
    Platform(0xA2C7, BDF(0x00, 0x16, 0x00), PCI_ACCESS, SPS40),
    Platform(0xA2C8, BDF(0x00, 0x16, 0x00), PCI_ACCESS, SPS40),

    # SPS 5.0
    Platform(0xA300, BDF(0x00, 0x16, 0x00), PCI_ACCESS, SPS50),
    Platform(0xA301, BDF(0x00, 0x16, 0x00), PCI_ACCESS, SPS50),
    Platform(0xA302, BDF(0x00, 0x16, 0x00), PCI_ACCESS, SPS50),
    Platform(0xA303, BDF(0x00, 0x16, 0x00), PCI_ACCESS, SPS50),
    Platform(0xA304, BDF(0x00, 0x16, 0x00), PCI_ACCESS, SPS50),
    Platform(0xA305, BDF(0x00, 0x16, 0x00), PCI_ACCESS, SPS50),
    Platform(0xA306, BDF(0x00, 0x16, 0x00), PCI_ACCESS, SPS50),
    Platform(0xA307, BDF(0x00, 0x16, 0x00), PCI_ACCESS, SPS50),
    Platform(0xA308, BDF(0x00, 0x16, 0x00), PCI_ACCESS, SPS50),
    Platform(0xA309, BDF(0x00, 0x16, 0x00), PCI_ACCESS, SPS50),
    Platform(0xA30A, BDF(0x00, 0x16, 0x00), PCI_ACCESS, SPS50),
    Platform(0xA30B, BDF(0x00, 0x16, 0x00), PCI_ACCESS, SPS50),
    Platform(0xA30C, BDF(0x00, 0x16, 0x00), PCI_ACCESS, SPS50),
    Platform(0xA30D, BDF(0x00, 0x16, 0x00), PCI_ACCESS, SPS50),
    Platform(0xA30E, BDF(0x00, 0x16, 0x00), PCI_ACCESS, SPS50),
    Platform(0xA30F, BDF(0x00, 0x16, 0x00), PCI_ACCESS, SPS50),
    Platform(0xA310, BDF(0x00, 0x16, 0x00), PCI_ACCESS, SPS50),
    Platform(0xA311, BDF(0x00, 0x16, 0x00), PCI_ACCESS, SPS50),
    Platform(0xA312, BDF(0x00, 0x16, 0x00), PCI_ACCESS, SPS50),
    Platform(0xA313, BDF(0x00, 0x16, 0x00), PCI_ACCESS, SPS50),
    Platform(0xA314, BDF(0x00, 0x16, 0x00), PCI_ACCESS, SPS50),
    Platform(0xA315, BDF(0x00, 0x16, 0x00), PCI_ACCESS, SPS50),
    Platform(0xA316, BDF(0x00, 0x16, 0x00), PCI_ACCESS, SPS50),
    Platform(0xA317, BDF(0x00, 0x16, 0x00), PCI_ACCESS, SPS50),
    Platform(0xA318, BDF(0x00, 0x16, 0x00), PCI_ACCESS, SPS50),
    Platform(0xA319, BDF(0x00, 0x16, 0x00), PCI_ACCESS, SPS50),
    Platform(0xA31A, BDF(0x00, 0x16, 0x00), PCI_ACCESS, SPS50),
    Platform(0xA31B, BDF(0x00, 0x16, 0x00), PCI_ACCESS, SPS50),
    Platform(0xA31C, BDF(0x00, 0x16, 0x00), PCI_ACCESS, SPS50),
    Platform(0xA31D, BDF(0x00, 0x16, 0x00), PCI_ACCESS, SPS50),
    Platform(0xA31E, BDF(0x00, 0x16, 0x00), PCI_ACCESS, SPS50),
    Platform(0xA31F, BDF(0x00, 0x16, 0x00), PCI_ACCESS, SPS50),
]

def get_platform():
    ''' searches for matching LPC Device ID '''
    for bus in range(0, 256):
        val = pci.read_pci_register(bus, LPC_PCI_DEVICEID, LPC_PCI_FUNCTION, 0)
        for platform in SUPPORTED_PLATFORMS:
            searched_dev = platform.pch_did << 16 | VENDOR_ID_INTEL
            if val == searched_dev:
                platform.heci_bdf.bus = bus
                return platform

    raise sps.exceptions.PlatformUnknown

def reveal_heci(platform):
    ''' reveals HECI-1 device if possible '''
    hide_on_exit = False
    status = True

    # older platforms
    if platform.sps_gen == SPS30:
        bus = platform.heci_bdf.bus
        dev = LPC_PCI_DEVICEID
        funct = LPC_PCI_FUNCTION
        rcba = pci.read_pci_register(bus, dev, funct, LPC_RCBA_REG_OFFSET)
        if not rcba & 0x1:
            return False, False
        rcba_base_addr = (rcba & RCBA_BASE_ADDR_MASK) + HECI_HOST_ENABLE_RCRB_OFFSET
        data = pci.mem_io_read_32(rcba_base_addr)
        if data & HECI_ENABLEMENT_A1B0_BITMASK:
            data &= ~HECI_ENABLEMENT_A1B0_BITMASK
            pci.mem_io_write_32(rcba_base_addr, data)
            if pci.mem_io_read_32(rcba_base_addr) != data:
                return False, False
            hide_on_exit = True

    # SPS 4.0 nd newer
    else:
        bus = platform.heci_bdf.bus
        dev = platform.heci_bdf.dev
        funct = platform.heci_bdf.funct
        heci1_cmd_register = 0
        if platform.heci_access == PCIe_ACCESS:
            heci1_cmd_register = pci.read_register_via_pcie(bus, dev, funct, COMMAND_REGISTER_OFFSET)
        else:
            heci1_cmd_register = pci.read_pci_register(bus, dev, funct, COMMAND_REGISTER_OFFSET)

        if not heci1_cmd_register or heci1_cmd_register == 0xFFFFFFFF:
            return False, False

        if not heci1_cmd_register & MEMORY_SPACE_ENABLE_MASK:
            heci1_cmd_register |= MEMORY_SPACE_ENABLE_MASK
            if platform.heci_access == PCIe_ACCESS:
                pci.write_register_via_pcie(bus, dev, funct, COMMAND_REGISTER_OFFSET, heci1_cmd_register)
                heci1_cmd_register = pci.read_register_via_pcie(bus, dev, funct, COMMAND_REGISTER_OFFSET)
            else:
                pci.write_pci_register(bus, dev, funct, COMMAND_REGISTER_OFFSET, heci1_cmd_register)
                heci1_cmd_register = pci.read_pci_register(bus, dev, funct, COMMAND_REGISTER_OFFSET)

            if heci1_cmd_register & MEMORY_SPACE_ENABLE_MASK:
                hide_on_exit = True
            else:
                return False, False

    return status, hide_on_exit

def hide_heci(platform):
    ''' hides HECI-1 device '''
    status = True

    # older platforms
    if platform.sps_gen == SPS30:
        bus = platform.heci_bdf.bus
        dev = LPC_PCI_DEVICEID
        funct = LPC_PCI_FUNCTION
        rcba = pci.read_pci_register(bus, dev, funct, LPC_RCBA_REG_OFFSET)
        if not rcba & 0x1:
            return False
        rcba_base_addr = (rcba & RCBA_BASE_ADDR_MASK) + HECI_HOST_ENABLE_RCRB_OFFSET
        data = pci.mem_io_read_32(rcba_base_addr) | HECI_ENABLEMENT_A1B0_BITMASK
        pci.mem_io_write_32(rcba_base_addr, data)
        if pci.mem_io_read_32(rcba_base_addr) != data:
            return False

    # SPS 4.0 and newer
    else:
        bus = platform.heci_bdf.bus
        dev = platform.heci_bdf.dev
        funct = platform.heci_bdf.funct
        heci1_cmd_register = 0
        if platform.heci_access == PCIe_ACCESS:
            heci1_cmd_register = pci.read_register_via_pcie(bus, dev, funct, COMMAND_REGISTER_OFFSET)
        else:
            heci1_cmd_register = pci.read_pci_register(bus, dev, funct, COMMAND_REGISTER_OFFSET)

        if not heci1_cmd_register or heci1_cmd_register == 0xFFFFFFFF:
            return False

        if heci1_cmd_register & MEMORY_SPACE_ENABLE_MASK:
            heci1_cmd_register &= ~MEMORY_SPACE_ENABLE_MASK
            if platform.heci_access == PCIe_ACCESS:
                pci.write_register_via_pcie(bus, dev, funct, COMMAND_REGISTER_OFFSET, heci1_cmd_register)
                heci1_cmd_register = pci.read_register_via_pcie(bus, dev, funct, COMMAND_REGISTER_OFFSET)
            else:
                pci.write_pci_register(bus, dev, funct, COMMAND_REGISTER_OFFSET, heci1_cmd_register)
                heci1_cmd_register = pci.read_pci_register(bus, dev, funct, COMMAND_REGISTER_OFFSET)

            if heci1_cmd_register & MEMORY_SPACE_ENABLE_MASK:
                return False

    return status
